#!/bin/bash

if [ -z "$BASH_VERSION" ]; then
	echo >&2 "This script requires 'bash' shell"
	exit 1
fi

if [ "$(echo -e TEST)" = '-e TEST' ]; then
	IS_POSIX=1
fi

LP="$PWD"
D_VER='head'

fecho()
{
	if [ "$1" = 'COL' ]; then
		shift
		color="$1"
		shift
	fi
	if [ -z "$IS_POSIX" ]; then
		echo -e "\\033[${color}m$*\\033[0m" | tee -a "$LP/paragon-ufsd-install.log"
	else
		echo "$*" | tee -a "$LP/paragon-ufsd-install.log"
	fi
}

fancy()
{
	echo '=======\/\/\/=======' "$*" '=======\/\/\/=======' >> "$LP/paragon-ufsd-install.log"
}

print_version_error()
{
	fecho COL 31 "Current kernel version is out of supported range. Proceed at your own risk? [Yes/no]"
	read -r answer
	case "$answer" in
	[nN]|[nN][oO])
		exit 1
		;;
	esac
}

if echo "$LP" | grep -q :; then
	fecho COL 31 "Path to current work directory $LP contains invalid character."
	fecho COL 31 "Installation was aborted."
	exit 1
fi

run_install_driver()
{
	fecho COL 31 "Continue installing? [yes/no/read]."
	read -r answer
	case "$answer" in
	[yY]|[yY][eE][sS])
		;;
	[rR]|[rR][eE][aA][dD])
		less License.txt
		run_install_driver
		return $?
		;;
	*)
		return 1
		;;
	esac

	echo "================= UFSD install log ===================" > "$LP/paragon-ufsd-install.log"
	echo "Start: $(date)" >> "$LP/paragon-ufsd-install.log"
	echo "OS Version: $(uname -a)" >> "$LP/paragon-ufsd-install.log"

	# check kernel version
	# define available versions
	min_major=2
	min_minor=6
	min_patch=36

	max_major=4
	max_minor=12
	max_patch=5

	# get current version
	curr_major="$(uname -r | cut -d'-' -f1 | cut -d'.' -f1)"
	curr_minor="$(uname -r | cut -d'-' -f1 | cut -d'.' -f2)"
	curr_patch="$(uname -r | cut -d'-' -f1 | cut -d'.' -f3)"

	# compare versions
	if ! [ "$min_major" -lt "$curr_major" ]; then
		if ! { [ "$min_major" -eq "$curr_major" ] && [ "$min_minor" -lt "$curr_minor" ]; }; then
			if ! { [ "$min_minor" -eq "$curr_minor" ] && [ "$min_patch" -le "$curr_patch" ]; }; then
				print_version_error
			fi
		fi
	fi

	if ! [ "$max_major" -gt "$curr_major" ]; then
		if ! { [ "$max_major" -eq "$curr_major" ] && [ "$max_minor" -gt "$curr_minor" ]; }; then
			if ! { [ "$max_minor" -eq "$curr_minor" ] && [ "$max_patch" -ge "$curr_patch" ]; }; then
				print_version_error
			fi
		fi
	fi

	# end check kernel version

	fecho COL 32 "Searching and removing previously installed UFSD driver in /lib/modules/$(uname -r)/"
	find "/lib/modules/$(uname -r)" \( -name jnl.ko -o -name ufsd.ko \) -delete

	rmmod ufsd jnl 2>/dev/null

	if ! [ -d util/ ]; then
		touch express.version
	fi

	#################################################################################################
	# Next options such as dkms support and additional utilities are available in Professional      #
	# version only.                                                                                 #
	# Please consider buying Paragon NTF/HFS+ for Linux Professional if you want to make them work. #
	# http://www.paragon-software.com/home/ntfs-linux-professional/                                 #
	#################################################################################################

	if [ -f util/paragon-dkms.conf ]; then
		fecho COL 33 "Would you like UFSD driver to rebuild after kernel updates? [yes/no]"
		read -r answer
		case "$answer" in
		[yY]|[yY][eE][sS])

			if ! which dkms > /dev/null; then
				fecho COL 32 "Error: please install dkms package first"
				exit 1
			else
				rm -rf "/usr/src/paragon-ufsd-${D_VER}"
				rm -rf "/var/lib/dkms/paragon-ufsd/"

				if [ -x "$(which systemctl)" ]; then
					if ! systemctl status dkms_autoinstaller 2>/dev/null | grep -q active && \
						! systemctl status dkms.service 2>/dev/null | grep -q active; then
						fecho COL 31 "Service dkms_autoinstaller/dkms.service is disabled or not installed. Driver may not rebuild with kernel updates."
						fecho COL 31 "Please run 'insserv /usr/lib/dkms/dkms_autoinstaller' to install and/or"
						fecho COL 31 "'systemctl start dkms_autoinstaller'/'systemctl start dkms.service' to enable it."
					fi
				fi

				fecho COL 32 "Setting DKMS configuration"
				mkdir "/usr/src/paragon-ufsd-${D_VER}/"
				cp -r ./ "/usr/src/paragon-ufsd-${D_VER}/"
				mv "/usr/src/paragon-ufsd-${D_VER}/util/paragon-dkms.conf" "/usr/src/paragon-ufsd-${D_VER}/dkms.conf"
				dkms add -m paragon-ufsd -v "${D_VER}" >> "$LP/paragon-ufsd-install.log"

				fecho COL 32 "Preparing to install"
				dkms build -m paragon-ufsd -v "${D_VER}" >> "$LP/paragon-ufsd-install.log"
				dkms_ret=$?

				fancy 'DKMS Configure log'
				cat "/var/lib/dkms/paragon-ufsd/${D_VER}/$(uname -r)/$(uname -m)/log/make.log" >> "$LP/paragon-ufsd-install.log"
				fancy 'DKMS Configure log'

				if [ $dkms_ret -ne 0 ]; then
					fecho COL 31 "Can't prepare driver configuration"
					exit 1
				fi

				fecho COL 32 "Building and installing driver to kernel $(uname -r)"
				if ! dkms install --force -m paragon-ufsd -v "${D_VER}" >> "$LP/paragon-ufsd-install.log"; then
					fecho COL 31 "Can't build driver"
					exit 1
				fi

				fecho COL 32 "Driver was installed to system"
			fi
			;;
		*)
			touch no_dkms.version
			;;
		esac
	fi

	if [ -f express.version ] || [ -f no_dkms.version ]; then
		fecho COL 32 "Preparing to install"

		[ -f Makefile ] && make clean >> "$LP/paragon-ufsd-install.log"

		if ! ./configure >> "$LP/paragon-ufsd-install.log" 2>&1; then
			fecho COL 31 "Can't prepare driver configuration"
			fancy 'Configure log'
			cat "config.log" >> "$LP/paragon-ufsd-install.log"
			fancy 'Configure log'
			exit 1
		fi

		fecho COL 32 "Building driver to kernel $(uname -r)"

		if ! make driver >> "$LP/paragon-ufsd-install.log" 2>&1; then
			fecho COL 31 "Can't build driver"
			exit 1
		fi

		fecho COL 32 "Install driver to kernel $(uname -r)"

		if ! make driver_install >> "$LP/paragon-ufsd-install.log" 2>&1; then
			fecho COL 31 "Can't install driver"
			exit 1
		fi

		fecho COL 32 "Driver was installed to system"
	fi

	if [ -x "$(which systemctl)" ]; then
		fecho COL 32 "Setting driver autoload at system startup"
		rm -f /etc/modules-load.d/paragon-ufsd.conf
		if ! [ -f express.version ]; then
			echo jnl > /etc/modules-load.d/paragon-ufsd.conf
		fi
		echo ufsd >> /etc/modules-load.d/paragon-ufsd.conf
	fi

	if [ -f /etc/modules ] && ! grep -q ufsd /etc/modules; then
		fecho COL 32 "Setting driver autoload at system startup"
		if ! [ -f express.version ]; then
			echo jnl >> /etc/modules
		fi
		echo ufsd >> /etc/modules
	fi

	if [ -f util/mount.paragon-ufsd ]; then
		fecho COL 33 "Would you like to mount NTFS/HFS+ volumes with UFSD driver automatically? [yes/no]"
		read -r answer
		case "$answer" in
		[yY]|[yY][eE][sS])
			cp util/mount.paragon-ufsd  /usr/sbin/
			chown root:root /usr/sbin/mount.paragon-ufsd
			chmod 04755 /usr/sbin/mount.paragon-ufsd

			paths="$(readlink -e /sbin)"$'\n'"$(readlink -e /usr/sbin)"

			for p in $(echo "$paths" | sort -u); do
				for fs in mount.ntfs mount.hfsplus; do
					if [ -f "$p/$fs" ] && [ ! "$(readlink -e "$p/$fs")" = '/usr/sbin/mount.paragon-ufsd' ]; then
						mv -n "$p/$fs" "$p/$fs.bak"
					fi
					ln -sf /usr/sbin/mount.paragon-ufsd "$p/$fs"
				done
			done
			fecho COL 32 "Automount configured"
			;;
		esac
	fi

	if [ -d linutil ]; then
		fecho COL 33 "Would you like to install NTFS/HFS+ utilites? [yes/no]"
		read -r answer
		case "$answer" in
		[yY]|[yY][eE][sS])
			if [ -d "/usr/src/paragon-ufsd-${D_VER}/linutil/" ]; then
				cd "/usr/src/paragon-ufsd-${D_VER}/linutil/" || exit 1
			else
				cd "linutil/" || exit 1
			fi

			fecho COL 32 "Making NTFS/HFS+ utilites"

			if ! make retail_util >> "$LP/paragon-ufsd-install.log" 2>&1; then
				fecho COL 31 "Can't make utilites"
				exit -1
			fi

			fecho COL 32 "Installing NTFS/HFS+ utilites"

			if ! make install_retail >> "$LP/paragon-ufsd-install.log" 2>&1; then
				fecho COL 31 "Can't install utilites"
				exit -1
			fi

			fecho COL 32 "Utilites installed"
			cd "$LP" || exit 1
			;;
		esac
	fi

	fecho COL 32 "Installation complete!"
}

if [ "$(id -u)" -ne 0 ]; then
	fecho COL 31 "Not enough permissions to install. Please login as root."
	exit 1
else
	fecho COL 31 "By installing this software you accept the terms of End User License Agreement listed in License file."
	run_install_driver
fi

exit 0

